@@ -1,16 +1,32 @@
 import requests
 
 from exceptions import *
+from signed_request import parse_signed_request
 
 try:
-    import json
-except ImportError:
     import simplejson as json
+except ImportError:
+    import json
 
 class GraphAPI(object):
     
-    def __init__(self, oauth_token=None):
-        self.oauth_token = oauth_token
+    def __init__(self, oauth_token=None, signed_request=None, app_secret=None):
+        """Initialize GraphAPI with an oauth_token, signed request or neither.
+
+        If signed_request is given along with app_secret, oauth_token will be
+        extracted automatically from the signed_request.
+
+        Arguments:
+        oauth_token -- OAuth 2.0 token
+        signed_request -- raw signed_request taken from POST (optional)
+        app_secret -- Application's app_secret required to parse signed_request (optional)
+        """
+
+        if not oauth_token and signed_request and app_secret:
+            sr = parse_signed_request(signed_request, app_secret)
+            self.oauth_token = sr.get("oauth_token", None)
+        else:
+            self.oauth_token = oauth_token
         
     def get(self, path='', **options):
         """
@@ -98,12 +114,11 @@ class GraphAPI(object):
         # Convert option lists to comma-separated values; Facebook chokes on array-like constructs
         # in the query string (like [...]?ids=['johannes.gorset', 'atle.mo']).
         for key, value in data.items():
-            if type(value) is list and all([type(item) is str for item in value]):
+            if type(value) is list and all([type(item) in (str, unicode) for item in value]):
                 data[key] = ','.join(value)
         
         if self.oauth_token:
             data.update({'access_token': self.oauth_token })
-        
         response = requests.request(method, 'https://graph.facebook.com/%s' % path, data=data)
 
         return self._parse(response.content)
@@ -150,3 +165,4 @@ class GraphAPI(object):
 
     class Error(FacepyError):
         pass
+
